<script setup>
import Highcharts from 'highcharts';
import DK_Input from '@/components/dk_Input.vue';
import { Link } from '@inertiajs/inertia-vue3';
import DK_Button from '@/components/dk_Button.vue';
import Datepicker from '@vuepic/vue-datepicker';
import '@vuepic/vue-datepicker/dist/main.css';
import { startOfDay, endOfDay, subDays, endOfMonth, endOfYear, startOfMonth, startOfYear, subMonths } from 'date-fns';
import { axiosApiCalls } from '../../../public/libs/datakorumJS';
import JetModal from '@/Jetstream/Modal.vue';

let AppLayout;

if (window.location.origin.indexOf('localhost:8000') !== -1 || window.location.origin.indexOf('water.baltoro.app') !== -1) {
    AppLayout = require('@/Layouts/AppLayoutWater.vue').default;
} else {
    AppLayout = require('@/Layouts/AppLayoutCity.vue').default;
}

</script>

<style>

table.dataTable thead th,
table.dataTable thead td {
    padding: 5px 10px;
    font-size: 0.9rem !important;
    border-bottom: 1px solid rgba(200, 200, 200, 0.75) !important;
}

.dataTables_wrapper .dataTables_paginate .paginate_button.disabled,
.dataTables_wrapper .dataTables_paginate .paginate_button.disabled:hover,
.dataTables_wrapper .dataTables_paginate .paginate_button.disabled:active {
    cursor: default;
    color: rgba(200, 200, 200, 0.9) !important;
    border: 1px solid transparent;
    background: transparent;
    box-shadow: none;
}

.dataTables_wrapper .dataTables_paginate .paginate_button {
    box-sizing: border-box;
    display: inline-block;
    min-width: 1.5em;
    padding: 0.5em 1em;
    margin-left: 2px;
    text-align: center;
    text-decoration: none !important;
    cursor: pointer;
    cursor: hand;
    color: rgba(200, 200, 200, 0.9) !important;
    border: 0;
    border-radius: 0.375rem;
}

.dataTables_wrapper .dataTables_paginate .paginate_button:hover {
    color: rgba(200, 200, 200, 1) !important;
    background-color: rgba(0, 152, 219, 0.1) !important;
    background-image: unset !important;
    border: 0;
}

input[type="number"]::-webkit-inner-spin-button,
input[type="number"]::-webkit-outer-spin-button {
  -webkit-appearance: none;
  margin: 0;
}

.circle{
  background: transparent;
  overflow:hidden;
}

.wave{
  position:relative;
  width:100%;
  height:100%;
  background:#00569C;
  border-radius:50%;
  box-shadow:inset 0 0 50px rgba(0,0,0,.5);
}
.wave:before,.wave:after{
  content:'';
  position:absolute;
  width:200%;
  height:200%;
  top:0;
  left:50%;
  transform:translate(-60%,-85%);
  background:transparent;
}
.wave:before{
  border-radius:45%;
  background:rgba(255,255,255,0.2);
  animation:animateWave 10s linear infinite;
}
.wave:after{
  border-radius:40%;
  background:rgba(255,255,255,.1);
  animation:animateWave 20s linear infinite;
}
@keyframes animateWave{
  0%{
     transform:translate(-50%,-75%) rotate(0deg);
  }
  100%{
     transform:translate(-50%,-75%) rotate(360deg);
  }
}

.breeze{
  position:relative;
  width:100%;
  height:100%;
  background:rgba(255,255,255,0.3);
  border-radius:75%;
}
.breeze:before,.breeze:after{
  content:'';
  position:absolute;
  width:100%;
  height:100%;
  top:50%;
  left:75%;
  transform:translate(-60%,-85%);
  background:transparent;
}
.breeze:before{
  border-radius: 60% 50% 45% 52.5%;
  background:rgba(255,255,255,0.3);
  animation:animateBreeze 12.5s linear infinite;
}
.breeze:after{
  border-radius: 47.5% 40% 50% 65%;
  background:rgba(255,255,255,0.3);
  animation:animateBreeze 15s linear infinite;
}
@keyframes animateBreeze{
  0% {
    transform: translate(-75%,-50%) rotate(0deg);
  }
  25% {
    transform: translate(-82.5%,-46%) rotate(110deg);
  }
  50% {
    transform: translate(-85%,-53%) rotate(180deg);
  }
  75% {
    transform: translate(-77.5%,-56%) rotate(250deg);
  }
  100% {
    transform: translate(-75%,-50%) rotate(360deg);
  }
}

.dp__action_row {
    border-radius: 0.5rem;
}
.dp__input_reg {
    background: none;
    border-width: 1px;
    border-color: rgba(200, 200, 200, 0.9);
    border-radius: 0.5rem;
}
.dp__pointer {
    padding-top: 0.25rem;
    padding-bottom: 0.25rem;
}
.dp__theme_light {
    --dp-background-color: rgba(0, 57, 93, 0.95);
    --dp-text-color: #e3e3e3;
    --dp-hover-color: rgba(0, 255, 255, 0.2);
    --dp-hover-text-color: #FFFAFA;
    --dp-hover-icon-color: #e3e3e3;
    --dp-primary-color: rgba(0, 255, 255, 0.5);
    --dp-primary-text-color: #FFFAFA;
    --dp-secondary-color: #959595;
    --dp-border-color: #FFFAFA;
    --dp-menu-border-color: #FFFAFA;
    --dp-border-color-hover: #FFFAFA;
    --dp-disabled-color: #959595;
    --dp-scroll-bar-background: #f3f3f3;
    --dp-scroll-bar-color: #959595;
    --dp-success-color: rgba(5, 255, 243, 0.7);
    --dp-success-color-disabled: rgba(5, 255, 243, 0.7);
    --dp-icon-color: #e3e3e3;
    --dp-danger-color: #e6002e;
    --dp-highlight-color: rgba(25, 118, 210, 0.1);
}

input[type="number"] {
  width: 100%;
  border-bottom: 1px solid rgba(175, 175, 175, 0.5);
  transition: border-color 0.3s ease;
  outline: none;
  background-color: transparent;
  width: 1.75rem;
  text-align: center;
  -webkit-appearance: none;
  margin: 0;
}

input[type="number"]:focus {
  border-color: rgba(225, 225, 225, 0.7);
}

input[type="number"]::-webkit-inner-spin-button,
input[type="number"]::-webkit-outer-spin-button {
  -webkit-appearance: none;
  margin: 0;
}
</style>

<template>
    <AppLayout title="Valve">
        <div style="height: calc(100vh - 4.75rem);" class="bg-darkblue-background -z-1 overflow-y-scroll cursor-default">
            <div class="mx-6 mt-2 text-xs text-center text-gray-light font-light flex justify-between">
                <div class="grid place-content-center justify-start justify-items-start">
                    <div class="inline-flex justify-start items-center py-2 gap-4">
                        <div class="flex flex-row">
                            <p class="text-white-snow text-2xl w-full text-left whitespace-nowrap">{{ this.data.device.name }}&nbsp;</p>
                            <p v-if="this.adminLvl('admin')" class="text-white-snow text-2xl w-full text-left whitespace-nowrap">(Id:&nbsp;{{ this.data.device.id }})</p>
                        </div>
                    </div>
                    <div class="flex flex-row gap-4 justify-center items-center">
                        <Datepicker id="interval_date" class="min-w-[21rem]" v-model="date" vertical
                            :presetRanges="this.presetRanges" range @update:modelValue="methodUpdateDate" :clearable="false"
                            :format="this.methodFormatDatetoDatepicker"></Datepicker>

                        <DK_Button color="blue" :fixedSize="false" width="fit" v-on:click="this.openDistances()" v-if="this.adminLvl('admin')">
                            {{ this.$t("general.Edit") +' '+this.$t("general.Distance").toLowerCase() + 's' }}
                        </DK_Button>

                        <DK_Button color="blue" :fixedSize="false" width="fit" v-on:click="this.openFastMode()" v-if="this.adminLvl('admin')">
                            {{ this.$t("general.Fast mode") }}
                        </DK_Button>
                    </div>
                </div>

                <div class="pt-6 grid justify-end items-end">
                    <div class="flex justify-center items-center gap-2">
                        <!-- <Link onclick="window.emitter.emit('changeLoaderStatus', true, 'orbit')" :href="route('cityReadings', { idDevice: this.data.device.id })">
                            <DK_Button color="blue" :fixedSize="true">
                                {{ this.$t("general.Readings") }}
                            </DK_Button>
                        </Link> -->

                        <DK_Button color="blue" :fixedSize="true" onclick="window.emitter.emit('changeLoaderStatus', true, 'orbit'); window.history.go(-1)">
                            {{ this.$t("general.Back") }}
                        </DK_Button>
                    </div>
                </div>
            </div>

            <div class="px-6 py-4 h-fit w-full gap-4 grid"
                v-bind:class="{'2xl:grid-cols-4 grid-cols-5 grid-rows-4 2xl:grid-rows-5' : this.adminLvl('user'), '2xl:grid-cols-4 grid-cols-5 grid-rows-6' : this.adminLvl('admin')}">
                <div class="2xl:col-span-2 col-span-3 row-span-2 col-start-1 2xl:col-end-3 col-end-4 row-start-1 row-end-3 flex flex-row justify-items-stretch items-stretch gap-4 h-fit">
                    <div class="w-full gap-x-4 gap-y-4 flex flex-col items-center justify-center">
                        <div class="w-full h-full baltoro-container py-2 px-4 flex flex-col justify-evenly items-center text-white-snow text-base gap-2">
                            <div class="w-full flex justify-between items-center gap-2">
                                <p class="rounded-lg py-0.5 px-1.5 text-lg text-center whitespace-nowrap"
                                    v-bind:class="{ 'font-medium text-white-snow border-0 bg-gradient-to-r from-success-start to-success-end' : idColor == 0,
                                                    'font-medium text-white-snow border-0 bg-gradient-to-r from-info-start to-info-end' : idColor == 1,
                                                    'font-medium text-white-snow border-0 bg-gradient-to-r from-alert-start to-alert-end' : idColor == 2}"
                                    >{{ this.$t('general.' + this.getStateName()) }}</p>

                                <div class="flex items-center justify-end w-min h-min" v-if="this.adminLvl('admin')">
                                    <!-- Battery -->
                                    <i v-if="this.data.device.Dparameters.Battery <= 2900" class="DK-Batt-25 animate-pulse"
                                        style="font-size:1.25rem; color:#d1d1d1; margin-right:0rem;  margin-left:0.5rem; position:relative; top:10%"></i>
                                    <i v-else-if="this.data.device.Dparameters.Battery <= 3100" class="DK-Batt-50"
                                        style="font-size:1.25rem; color:#d1d1d1; margin-right:0rem; margin-left:0.5rem; position:relative; top:10%"></i>
                                    <i v-else-if="this.data.device.Dparameters.Battery <= 3300" class="DK-Batt-75"
                                        style="font-size:1.25rem; color:#d1d1d1; margin-right:0rem; margin-left:0.5rem; position:relative; top:10%"></i>
                                    <i v-else class="DK-Batt-100"
                                        style="font-size:1.25rem; margin-right:0rem; color:#d1d1d1; margin-left:0.5rem; position:relative; top:10%"></i>

                                    <!-- Coverage SVG -->
                                    <i v-if="this.data.device.Dparameters.RSSI <= 110" class="DK-signal-01"
                                        style="font-size:1rem; margin-right:0rem; color:#d1d1d1; margin-left:0.5rem; position:relative; top:0%"></i>
                                    <i v-else-if="this.data.device.Dparameters.RSSI <= 100" class="DK-signal-02"
                                        style="font-size:1rem; margin-right:0rem; color:#d1d1d1; margin-left:0.5rem; position:relative; top:0%"></i>
                                    <i v-else-if="this.data.device.Dparameters.RSSI <= 85" class="DK-signal-03"
                                        style="font-size:1rem; margin-right:0rem; color:#d1d1d1; margin-left:0.5rem; position:relative; top:0%"></i>
                                    <i v-else-if="this.data.device.Dparameters.RSSI <= 70" class="DK-signal-04"
                                        style="font-size:1rem; margin-right:0rem; color:#d1d1d1; margin-left:0.5rem; position:relative; top:0%"></i>
                                    <i v-else class="DK-signal-05"
                                        style="font-size:1rem; margin-right:0rem; color:#d1d1d1; margin-left:0.5rem; position:relative; top:0%"></i>
                                </div>
                            </div>

                            <p class="text-justify text-sm text-gray-semilight w-full">{{ this.$t('text.' + this.getStateDescription()) }}</p>
                        </div>

                        <div class="baltoro-container px-4 py-2 w-full text-white-snow">

                            <div class="flex flex-col justify-start items-stretch text-sm gap-0 w-full">

                                <p class="text-lg text-cyan-text pb-2" v-if="!this.enableMaintButton">{{ this.$t('general.Records and Maintenance') }}</p>

                                <div class="flex flex-row justify-between w-full gap-6 whitespace-nowrap">
                                    <p class="font-light text-cyan-text">{{ this.$t('general.Last sent') + ': ' }}</p>
                                    <p class="w-fit text-right">{{ (this.data.dates != undefined && this.data.dates[0]) ? this.data.dates[0] : this.$t('general.Never') }}</p>
                                </div>

                                <div class="flex flex-row justify-between w-full gap-6 whitespace-nowrap">
                                    <p class="font-light text-cyan-text">{{ this.$t('general.Time elapsed') + ': ' }}</p>
                                    <p class="w-fit text-right">{{ this.elapsedTime }}</p>
                                </div>

                                <div class="flex justify-between w-full gap-2 whitespace-nowrap">
                                    <p class="font-light text-cyan-text">{{ this.$t('general.Upcoming maintenance') + ': ' }}</p>
                                    <div class="flex flex-row items-center justify-end" v-if="this.adminLvl('admin')">
                                        <DK_Input @keyup="this.enableMaintenanceButton()" type="number" min="2023" id="idYear" :value="this.maintenanceYear" textPosition="center" :border="false" width="2.5rem"></DK_Input>
                                        <p class="">-</p>
                                        <DK_Input @keyup="this.enableMaintenanceButton()" type="number" min="1" max="12" id="idMonth" :value="this.maintenanceMonth" textPosition="center" :border="false" width="1.2rem"></DK_Input>
                                        <p class="">-</p>
                                        <DK_Input @keyup="this.enableMaintenanceButton()" type="number" min="1" max="31" id="idDay" :value="this.maintenanceDay" textPosition="center" :border="false" width="1.2rem"></DK_Input>
                                    </div>
                                    <p v-else class="w-fit text-right">{{ this.maintenanceYear + '-' + this.maintenanceMonth + '-' + this.maintenanceDay }}</p>
                                </div>

                                <div class="w-full gap-2 flex justify-center pt-2" v-if="this.enableMaintButton && this.adminLvl('admin')">
                                    <DK_Button color="blue" :fixedSize="true" v-on:click="this.putMaintenanceDate()" >
                                        {{ this.$t("general.Update") }}
                                    </DK_Button>
                                    <DK_Button color="red" :fixedSize="true" v-on:click="this.cancelMaintenanceDate()" >
                                        {{ this.$t("general.Cancel") }}
                                    </DK_Button>
                                </div>
                            </div>
                        </div>
                    </div>

                    <div class="w-fit h-fit baltoro-container flex flex-row justify-between items-center gap-4 2xl:gap-8 px-4 2xl:px-6 pt-2 pb-2 relative">
                        <p class="absolute top-2 left-4 text-lg text-cyan-text pl-2">{{ this.$t('general.Current status') }}</p>
                        <div class="grid place-items-center mt-6 h-fit">
                            <img class="max-w-[125px] z-[20]" :src='"/assets/images/others/valve_states/" + this.getStateName() + ".png"'>
                        </div>
                        <div class="w-fit flex flex-row justify-center h-[250px] pr-[100px] pt-8">
                            <div class="flex flex-col w-[0.5rem] items-baseline justify-center h-full rounded-full">
                                <p class="text-xs text-center text-white-snow whitespace-nowrap pb-1">0 mm</p>
                                <div v-for="(segment, index) in (new Array(250))" :key="segment" class="w-full h-full relative"
                                    :style="{ backgroundColor: this.getDistanceColor(index) }">
                                    <div v-if="Math.round(this.data.data[3][0]) == index" class="absolute left-2 inset-y-0 flex items-center justify-center">
                                        <div class="border-b border-gray-semimedium h-0 w-2"></div>
                                        <div class="flex flex-col justify-center items-center px-1.5 py-0.5 rounded-md border border-gray-semimedium bg-blue-navbar">
                                            <div class="text-center flex justify-center items-baseline whitespace-nowrap gap-2">
                                                <p class="text-2xl font-sevensegment text-white-snow">{{ this.data.data[3][0] }}</p>
                                                <p class="text-md text-white-snow text-left">mm</p>
                                            </div>
                                            <div class="text-center flex justify-center items-baseline whitespace-nowrap gap-2">
                                                <p class="text-xs text-gray-light">{{ this.getDistanceRange(this.data.data[3][0]) }}</p>
                                            </div>
                                        </div>
                                    </div>
                                </div>
                                <p class="text-xs text-center text-white-snow whitespace-nowrap pt-1">250 mm</p>
                            </div>
                        </div>

                        <div class="flex flex-col justify-evenly items-center h-max gap-4 pt-2">

                            <div class="h-[5.5rem] w-[5.5rem] aspect-square grid place-items-center" id="myContainer">
                                <div class="grid place-items-center aspect-square rounded-full"
                                v-bind:class="{'circle' : this.data.data[4][0] == 0, 'border border-gray-semimedium' : this.data.data[4][0] != 1 }"
                                :style="'height:'+this.containerSize+'px; width:'+this.containerSize+'px;'">
                                    <div v-if="this.data.data[4][0] == 1" class="w-full h-full rounded-full grid place-items-center breeze">
                                        <p class="text-lg text-center text-white-snow">{{ this.$t('general.Air')}}</p>
                                    </div>
                                    <div v-else-if="this.data.data[4][0] == 0" class="w-full h-full rounded-full grid place-items-center wave">
                                        <p class="text-lg text-center text-white-snow">{{ this.$t('general.Water')}}</p>
                                    </div>
                                    <p v-else class="text-lg text-center text-white-snow">{{ this.$t('general.Unknown')}}</p>
                                </div>
                            </div>

                            <div class="w-[100px] flex flex-col justify-center items-center">
                                <div class="" id='id_chart_1'></div>
                                <div class="" id='id_label_1'></div>
                            </div>
                        </div>
                    </div>
                </div>

                <div class="2xl:col-span-2 baltoro-container p-4"
                    v-bind:class="{'row-span-4 row-end-5 col-span-2 col-start-4 2xl:col-start-3 row-start-1' : this.adminLvl('user'), 'col-start-1 2xl:col-end-3 col-end-4 row-start-3 col-span-3 row-span-4 row-end-7' : this.adminLvl('admin')}">
                    <table id="basicTable" class="row-border">
                        <thead class="bg-transparent text-gray-light">
                            <tr>
                                <th> {{ this.$t('general.Date') }} </th>
                                <th> {{this.data.names[2] + ( this.data.units[2] != '' ? ' (' + this.data.units[2] + ')' : '' )}} </th>
                                <th v-if="this.adminLvl('admin')"> {{this.data.names[3] + ( this.data.units[3] != '' ? ' (' + this.data.units[3] + ')' : '' )}} </th>
                                <th v-if="this.adminLvl('admin')"> {{this.data.names[4] + ( this.data.units[4] != '' ? ' (' + this.data.units[4] + ')' : '' )}} </th>
                                <th> {{ this.$t('general.State') }} </th>
                            </tr>
                        </thead>
                        <tbody class="text-center">
                        </tbody>
                    </table>
                </div>

                <div class="baltoro-container pt-6 pb-2 px-4 relative 2xl:h-full"
                    v-bind:class="{'col-start-1 row-start-3 col-end-4 2xl:col-end-3 row-end-5' : this.adminLvl('user'), 'row-start-1 row-end-3 col-span-2 row-span-2 col-start-4 2xl:col-start-3 col-end-6 ' : this.adminLvl('admin')}">
                    <p class="absolute top-2 left-4 text-lg text-cyan-text pl-2">{{ this.data.names[2] + ( this.data.units[2] != '' ? ' (' + this.data.units[2] + ')' : '' ) }}</p>
                    <div class="w-full h-full" id="id_chart__2"></div>
                </div>

                <div class="col-span-2 row-span-2 col-start-4 2xl:col-start-3 col-end-6 row-start-3 row-end-5 baltoro-container pt-6 pb-2 px-4 relative 2xl:h-full"
                    v-if="this.adminLvl('admin')">
                    <p class="absolute top-2 left-4 text-lg text-cyan-text pl-2">{{ this.data.names[3] + ( this.data.units[3] != '' ? ' (' + this.data.units[3] + ')' : '' ) }}</p>
                    <div class="w-full h-full" id="id_chart__3"></div>
                </div>

                <div class="col-span-2 row-span-2 col-start-4 2xl:col-start-3 col-end-6 row-start-5 row-end-7 baltoro-container pt-6 pb-2 px-4 relative 2xl:h-full"
                    v-if="this.adminLvl('admin')">
                    <p class="absolute top-2 left-4 text-lg text-cyan-text pl-2">{{ this.data.names[4] + ( this.data.units[4] != '' ? ' (' + this.data.units[4] + ')' : '' ) }}</p>
                    <div class="w-full h-full" id="id_chart__4"></div>
                </div>
            </div>
        </div>
        <JetModal :show="this.distancesModal" :closeable="false">
            <div class="px-6 py-2 cursor-default flex flex-col items-center w-full"  v-bind:class="{'w-[15rem]' : this.screen_width >= 768, 'w-[90vw]' : this.screen_width < 768}">

                <p class="font-light text-center text-white-snow text-xl pb-2">{{ this.$t("general.Edit") +' '+this.$t("general.Distance").toLowerCase() + 's' }}</p>

                <div class="flex flex-row items-center justify-between gap-2 w-full">
                    <p class="font-light text-cyan-text">{{ this.$t('general.dmin') }}: </p>
                    <div class="w-fit flex flex-row gap-1 items-center justify-center">
                        <DK_Input @keyup="this.enableDistanceButton()" type="number" id="idDmin" :value="this.dmin" textPosition="right" :border="false" width="10"></DK_Input>
                        <p class="font-light text-white-snow">mm</p>
                    </div>
                </div>

                <div class="flex flex-row items-center justify-between gap-2 w-full">
                    <p class="font-light text-cyan-text">{{ this.$t('general.dmax') }}: </p>
                    <div class="w-fit flex flex-row gap-1 items-center justify-center">
                        <DK_Input @keyup="this.enableDistanceButton()" type="number" id="idDmax" :value="this.dmax" textPosition="right" :border="false" width="10"></DK_Input>
                        <p class="font-light text-white-snow">mm</p>
                    </div>
                </div>

                <div class="flex flex-row items-center justify-between gap-2 w-full">
                    <p class="font-light text-cyan-text">{{ this.$t('general.Margin') }}: </p>
                    <div class="w-fit flex flex-row gap-1 items-center justify-center">
                        <DK_Input @keyup="this.enableDistanceButton()" type="number" id="idmargin" :value="this.margin" textPosition="right" :border="false" width="7"></DK_Input>
                        <p class="font-light text-white-snow">mm</p>
                    </div>
                </div>

                <div class="w-full flex gap-2 justify-center pt-4">
                    <DK_Button color="blue" :fixedSize="true" v-on:click="this.putDistances()" v-if="this.enableDistButton">
                        {{ this.$t("general.Update") }}
                    </DK_Button>
                    <DK_Button color="blue" :fixedSize="true" v-on:click="closeDistances()">
                        {{ this.$t("general.Cancel") }}
                    </DK_Button>
                </div>
            </div>
        </JetModal>

        <JetModal :show="this.fastMode" :closeable="false">
            <!-- <div class="px-6 py-2 cursor-default flex flex-col items-center w-full"  v-bind:class="{'w-[30rem]' : this.screen_width >= 768, 'w-[90vw]' : this.screen_width < 768}"> -->
            <div class="overflow-y-auto max-h-[80vh] pt-3 cursor-default" v-bind:class="{'w-[20rem]' : screen_width >= 768, 'w-[90vw]' : screen_width < 768}">
                <p class="font-light text-center text-white-snow text-xl pb-2">{{ this.$t("general.Edit") +' '+ this.$t("general.Fast mode").toLowerCase() }}</p>

                <div class="grid grid-cols-5 gap-x-8 px-8 py-4 gap-y-4 font-light text-gray-light text-center">

                    <div class="col-span-5">
                        <p class="text-white-snow text-left ml-1">{{ $t("general.Select day") }}</p>

                        <Datepicker
                            v-model="selectedDate"
                            :enable-time-picker="false"
                            :format="customFormat"
                        ></Datepicker>
                    </div>

                    <div class="col-span-5">
                        <p class="text-white-snow text-left ml-1">{{ $t("general.Select time range:") }}</p>

                        <div class="flex flex-row justify-start items-center gap-4">

                            <label>{{ this.$t("general.From") }}</label>

                            <div class="flex flex-row items-baseline gap-1">
                                <input type="number" v-model="this.start_at.hour" id="id_start_at" @input="validarRangoHorarioStart" max="23" min="0" required>
                                <label>: 00</label>
                            </div>

                            <label>{{ this.$t("general.Into").toLowerCase() }}</label>

                            <div class="flex flex-row items-baseline gap-1">
                                <input type="number" v-model="this.end_at.hour" id="id_end_at" @input="validarRangoHorarioEnd" max="23" min="0" required>
                                <label>: 00</label>
                            </div>

                        </div>
                    </div>
                </div>

                <div class="w-full flex gap-2 justify-center pt-4 pb-2">
                    <DK_Button color="blue" :fixedSize="true" v-on:click="this.putFastMode()" :disabled="this.disabledButtonEditFastMode">
                        {{ this.$t("general.Update") }}
                    </DK_Button>
                    <DK_Button color="blue" :fixedSize="true" v-on:click="closeFastMode()">
                        {{ this.$t("general.Cancel") }}
                    </DK_Button>
                </div>
            </div>
        </JetModal>
    </AppLayout>
</template>

<script>
export default {
    props: ['data', 'user'],

    data() {
        return {
            Back: localStorage.getItem('Back'),
            screen_width: 0,

            idState: undefined,
            idColor: undefined, // colores: [verde, azul, rojo]
            colors: ['#00b359', '#0070cc', '#bd1522'],

            sensorDefault: this.adminLvl('admin') ? [2, 3, 4] : 2,

            elapsedTime: '',
            containerSize: '0',
            date: undefined,

            maintenanceDay: String(this.data.device.descriptors.maintenanceDay ? this.data.device.descriptors.maintenanceDay : 1).padStart(2, '0'),
            maintenanceMonth: String(this.data.device.descriptors.maintenanceMonth ? this.data.device.descriptors.maintenanceMonth : 1).padStart(2, '0'),
            maintenanceYear: String(this.data.device.descriptors.maintenanceYear ? this.data.device.descriptors.maintenanceYear : 2000).padStart(4, '0'),

            enableMaintButton: false,
            enableDistButton:false,

            distancesModal: false,
            fastMode: false,
            selectedDate: null,
            disabledButtonEditFastMode: true,
            start_at: {
                hour: 0,
                min: 0,
            },
            end_at: {
                hour: 23,
                min: 59,
            },

            dmin: this.data.device.descriptors.dmin ? this.data.device.descriptors.dmin : 71,
            dmax: this.data.device.descriptors.dmax ? this.data.device.descriptors.dmax : 109,
            margin: this.data.device.descriptors.margin ? this.data.device.descriptors.margin : 5,

            convertDate(date) {
                var diasSemana = ['Sun_full', 'Mon_full', 'Tue_full', 'Wed_full', 'Thu_full', 'Fri_full', 'Sat_full'];
                var meses = ['Jan_full', 'Feb_full', 'Mar_full', 'Apr_full', 'May_full', 'Jun_full', 'Jul_full', 'Aug_full', 'Sep_full', 'Oct_full', 'Nov_full', 'Dec_full'];

                var fecha = new Date(date);

                var diaSemana = this.$t('general.' + diasSemana[fecha.getDay()]);
                var dia = fecha.getDate();
                var mes = this.$t('general.' + meses[fecha.getMonth()]);
                var anio = fecha.getFullYear();
                var hora = (fecha.getHours()).toString().length != 2 ? '0' + fecha.getHours() : fecha.getHours();
                var minutos = (fecha.getMinutes()).toString().length != 2 ? '0' + fecha.getMinutes() : fecha.getMinutes();
                var segundos = (fecha.getSeconds()).toString().length != 2 ? '0' + fecha.getSeconds() : fecha.getSeconds();

                if (this.user.language == 'es') {
                    var resultado = diaSemana + ' ' + dia + ' de ' + mes + ' ' + anio + ' a las ' + hora + ':' + minutos + ':' + segundos;
                } else if (this.user.language == 'en') {
                    var resultado = diaSemana + ' ' + dia + ' ' + mes + ' ' + anio + ' at ' + hora + ':' + minutos + ':' + segundos;
                }

                return resultado;
            },

            getElapsedTime: (fechaInicioStr, fechaFinStr) => {
                var diferenciaEnMilisegundos = 0;
                var disjoin = '';

                if (typeof fechaFinStr != 'undefined') {
                    const fechaInicio = new Date(fechaInicioStr);
                    const fechaFin = new Date(fechaFinStr);

                    diferenciaEnMilisegundos = fechaFin - fechaInicio;
                } else {
                    diferenciaEnMilisegundos = fechaInicioStr * 1000;
                }

                // Convertir la diferencia a días, horas, minutos y segundos
                const segundos = Math.floor(diferenciaEnMilisegundos / 1000);
                const minutos = Math.floor(segundos / 60);
                const horas = Math.floor(minutos / 60);
                const dias = Math.floor(horas / 24);

                // Crear un array para almacenar las partes no nulas del tiempo
                const partesTiempo = [];

                disjoin = ' ';
                if (dias > 0) partesTiempo.push(`${dias} d`);
                if (horas % 24 > 0) partesTiempo.push(`${horas % 24} h`);
                if (minutos % 60 > 0) partesTiempo.push(`${minutos % 60} min`);
                if (segundos % 60 > 0 && partesTiempo.length < 3) partesTiempo.push(`${segundos % 60} s`);

                // Unir las partes del tiempo en un string
                var tiempoFormateado = partesTiempo.join(disjoin);

                if (diferenciaEnMilisegundos == 0) {
                    tiempoFormateado = '0 segundos';
                }

                return tiempoFormateado;

            },

            formatDate: (date, utc) => {
                date = new Date(date);
                // utc en 0 para formatear hora local del usuario, 1 para formatear UTC
                if (utc == 0) { date.setHours(date.getHours() + this.user.timezone_offset) }
                else if (utc == -1) { date.setHours(date.getHours() - this.user.timezone_offset) }
                return (
                    date.getFullYear() + "-" +
                    ((date.getMonth() + 1 + '').length < 2 ? "0" + (date.getMonth() + 1) : (date.getMonth() + 1)) + "-" +
                    ((date.getDate() + '').length < 2 ? "0" + date.getDate() : date.getDate()) + " " +
                    ((date.getHours() + '').length < 2 ? "0" + date.getHours() : date.getHours()) + ":" +
                    ((date.getMinutes() + '').length < 2 ? "0" + date.getMinutes() : date.getMinutes()) + ":" +
                    ((date.getSeconds() + '').length < 2 ? "0" + date.getSeconds() : date.getSeconds())
                );
            },

            presetRanges: [
                { label: this.$t("general." + "Today"), range: [startOfDay(new Date()), new Date()] },
                { label: this.$t("general." + "Yesterday"), range: [startOfDay(subDays(new Date(), 1)), endOfDay(subDays(new Date(), 1))] },
                { label: this.$t("general." + "Last 7 Days"), range: [new Date().setDate(new Date().getDate() - 7), new Date().setDate(new Date().getDate())] },
                { label: this.$t("general." + "Last 14 Days"), range: [new Date().setDate(new Date().getDate() - 14), new Date().setDate(new Date().getDate())] },
                { label: this.$t("general." + "Last 30 Days"), range: [new Date().setDate(new Date().getDate() - 30), new Date().setDate(new Date().getDate())] },
                { label: this.$t("general." + "This month"), range: [startOfMonth(new Date()), new Date()] },
                { label: this.$t("general." + "Last month"), range: [startOfMonth(subMonths(new Date(), 1)), endOfMonth(subMonths(new Date(), 1))] }
            ],
        }
    },

    beforeCreate() {
        this.$i18n.locale = this.user.language
    },

    beforeMount() {
        this.screen_width = window.screen.width;

        this.idState = this.getState()

        this.setElapsedTime()
    },

    mounted() {

        let formatDate = this.formatDate

        this.Charts = {
            chart: {
                zoomType: 'x',
                backgroundColor: 'transparent',
                spacingTop: 20,
                animation: {
                    duration: 150
                },
                resetZoomButton: {
                    position: {
                        align: 'left',
                        verticalAlign: 'top',
                        x: 0,
                        y: 2.5
                    },
                    theme: {
                        r: 5.6,
                    }
                }
            },
            accessibility: {
                enabled: false
            },
            title: {
                text: ''
            },
            xAxis: {
                lineColor: '#647F8C',
                visible: true,
                labels: {
                    style: {
                        color: 'rgba(200, 200, 200, 0.9)',
                        opacity: 0.7,
                    }
                },
                type: "datetime",
                dateTimeLabelFormats: {
                    //millisecond: '%H:%M:%S.%L',
                    //second: '%H:%M:%S',
                    //minute: '%H:%M',
                    hour: '%H:%M',
                    day: '%m-%d',
                    week: '%m-%d',
                    month: '%Y-%m',
                    year: '%Y'
                },
                time: {
                    timezoneOffset: new Date().getTimezoneOffset(), // Configura la zona horaria actual
                    useUTC: false // Utiliza hora local
                }
            },
            yAxis: new Array(),
            exporting: {
                enabled: false
            },
            credits: {
                enabled: false
            },
            legend: {
                enabled: false,
                itemStyle: {
                    color: 'rgba(200, 200, 200, 0.9)',
                    opacity: 0.75,
                    fontWeight: 'light'
                },
                itemHoverStyle: {
                    color: 'rgba(200, 200, 200, 0.9)',
                    opacity: 1,
                    fontWeight: 'light'
                }
            },
            tooltip: {
                shared: true,
                split: false,
                backgroundColor: 'rgba(7,82,103,0.95)',
                borderRadius: 12,
                borderWidth: 1,
                borderColor: 'rgba(250, 250, 250, 0.65)',
                padding: 6,
                formatter: function () {
                    var s = '<b>' + '<p style="font-size:12px; line-height: 12px; font-weight: 300; color:rgba(250, 250, 250, 0.95)">' + formatDate(this.x, -1) + '</p><br/>';
                    $.each(this.points, function () {
                        if(typeof this.series.area != 'undefined'){
                            s += '<p style="font-size:12px; line-height: 12px; font-weight: 400; color:' + this.series.color + '">' + this.series.name + ': </p><p style="font-size:12px; line-height: 12px; font-weight: 400; color:' + this.series.color + '">' +
                            (parseFloat(this.y) == 0 ? 'Water' : (parseFloat(this.y) == 1 ? 'Air' : 'Undefined')) +
                            ' ' + this.series.userOptions.tooltip.valueSuffix + '</p><br/>';
                        }else{
                            s += '<p style="font-size:12px; line-height: 12px; font-weight: 400; color:' + this.series.color + '">' + this.series.name + ': </p><p style="font-size:12px; line-height: 12px; font-weight: 400; color:' + this.series.color + '">' + this.y +
                            ' ' + this.series.userOptions.tooltip.valueSuffix + '</p><br/>';
                        }
                    });

                    return s;
                },
            },
        };

        this.Gauge = {
            chart: {
                type: 'gauge',
                backgroundColor: "transparent",
                plotBackgroundColor: null,
                plotBackgroundImage: null,
                plotBorderWidth: 0,
                plotShadow: false,
                spacing: [0, 0, 0, 0], // Ajusta los márgenes del gráfico según tus necesidades
                height: 85,
                marginTop: -5,
                marginBottom: -20,
            },
            title: null,
            pane: {
                startAngle: -120,
                endAngle: 120,
                background: {
                backgroundColor: "transparent",
                borderWidth: 0
                }
            },
            tooltip: {
                enabled: false
            },
            xAxis: {
                title: {
                text: "",
                style: {
                    color: "#FFFAFA",
                    fontSize: "12px",
                    fontWeight: "normal",
                    fontStyle: "normal"
                }
                },
                labels: {
                    rotation: -45,
                    style: {
                        color: 'white',
                        fontSize: '10px' // Ajusta el tamaño de la fuente de las etiquetas según tus necesidades
                    }
                }
            },
            plotOptions: {
                gauge: {
                    borderWidth: 0,
                    borderColor: "#05fff3", // Cambia el color del borde de la serie a #05fff3
                    dataLabels: {
                        enabled: false, // Desactiva las etiquetas con el dato y su unidad
                    },
                    pivot: {
                        radius: "1%",
                        backgroundColor: "#fff", // Cambia el color del pivote a blanco
                    },
                    dial: {
                        backgroundColor: "#fff", // Cambia el color de la flecha a blanco
                        radius: "70%", // Ajusta el tamaño de la flecha según tus necesidades
                        baseWidth: 3, // Ajusta el ancho de la base de la flecha según tus necesidades
                        topWidth: 1, // Ajusta el ancho de la punta de la flecha según tus necesidades
                        baseLength: "10%", // Ajusta la longitud de la base de la flecha según tus necesidades
                        rearLength: "-20%", // Ajusta la longitud de la punta de la flecha según tus necesidades
                    }
                },
            },
            credits: {
                enabled: false
            },
            exporting: {
                enabled: false,
            }
        };

        let export_title = this.data.device.name + '_' + this.data.device.id + '_' + this.data.dates[0]

        this.table = $('#basicTable').DataTable({
            language: {
                url: '/assets/i18n/' + this.user.language + '.json'
            },
            order: [[0, "desc"]],
            dom: '<"row"<"col-sm-6"B><"col-sm-6">>t<"row"<"col-sm-6"i><"col-sm-6"p>><"clear">',
            ordering: true,
            pageLength: (this.screen_width <= 1280) ? ((this.adminLvl('admin')) ? 14 : 13) : ((this.adminLvl('admin')) ? 14 : 14),
            buttons: [
                {
                    extend: 'csv',
                    text: 'CSV',
                    title: export_title,
                    className: 'btn btn-brand',
                    exportOptions: {
                        columns: ':visible'
                    }
                },
                {
                    extend: 'excel',
                    text: 'Excel',
                    title: export_title,
                    className: 'btn btn-brand',
                    exportOptions: {
                        columns: ':visible'
                    }
                },
                {
                    extend: 'print',
                    text: 'Print',
                    title: export_title,
                    className: 'btn btn-brand',
                    exportOptions: {
                        columns: ':visible'
                    }
                },
            ]
        });

        window.emitter.emit('changeLoaderStatus', false, 'orbit')

        this.getContainerSize();
        this.rechargePage();

        setInterval(() => {
            this.setElapsedTime()
        }, 1000);
    },

    methods: {

        rechargePage() {
            this.date = [
                new Date(this.formatDate(this.data.dates[this.data.dates.length - 1], 1)),
                new Date(this.formatDate(this.data.dates[0], 1))
            ];

            require('highcharts/highcharts-more')(Highcharts);

            let min = 0;
            let max = 16;

            this.Gauge.yAxis= {
                min: min,
                max: max,
                tickPosition: 'outside',
                lineColor: '#05fff3',
                lineWidth: 1, // Ajusta el ancho de la línea del eje según tus necesidades
                minorTickPosition: 'outside',
                tickColor: '#05fff3',
                minorTickColor: '#05fff3',
                tickLength: 4, // Ajusta la longitud de las marcas del eje según tus necesidades
                minorTickWidth: 1,
                tickWidth: 1,
                minorTickLength: 3, // Ajusta la longitud de las marcas menores del eje según tus necesidades
                labels: {
                distance: 6, // Ajusta la distancia de las etiquetas al eje según tus necesidades
                rotation: 'auto',
                style: {
                    color: '#05fff3',
                    fontSize: '10px' // Ajusta el tamaño de la fuente de las etiquetas según tus necesidades
                }
                },
                tickPositions:
                [
                    min,
                    min + (max-min)/4,
                    min + (max-min)/2,
                    min + 3*(max-min)/4,
                    max
                ],
                offset: -10,
                endOnTick: false
            },

            this.Gauge.series = [{
                name: '',
                data: [{
                y:  this.data.data[2][0],
                }],
                tooltip: {
                valueSuffix: ''
                }
            }]

            if(document.getElementById("id_chart_1")) {
                Highcharts.chart("id_chart_1", this.Gauge);
            }

            var content =
            '<div class="content-between m-0 cursor-default text-base w-full pb-2">' +
                '<div class="text-center flex justify-center items-center w-full whitespace-nowrap gap-2 pl-4">' +
                    '<p class="text-2xl font-sevensegment text-white-snow">' + this.data.data[2][0] + '</p>' +
                    '<p class="text-md text-white-snow pb-0.5 self-end text-left">bar</p>' +
                '</div>' +
            '</div>'

            if(document.getElementById("id_label_1")) {
                document.getElementById("id_label_1").innerHTML = content;
            }

            this.table.clear();

            this.data.dates.forEach((date, index) => {

                let temp_row = ['<td>' + date + '</td>'];

                for (let sensor in this.data.data) {

                    if (typeof this.data.data[sensor][index] !== undefined && ( sensor == 2 || ((sensor == 3 && this.adminLvl('admin')) || (sensor == 4 && this.adminLvl('admin'))))) {
                        if (sensor == 4) {
                            temp_row.push('<td>' + (parseFloat(this.data.data[sensor][index]) == 0 ? 'Water' : (parseFloat(this.data.data[sensor][index]) == 1 ? 'Air' : 'Undefined')) + '</td>');
                        } else {
                            temp_row.push('<td>' + ((typeof this.data.data[sensor][index] != 'undefined') ? parseFloat(this.data.data[sensor][index]) : '') + '</td>');
                        }
                    }
                }

                let state = this.getStateName(index, true)
                let colorClasses = ''

                switch(state[1]){
                    case 0:
                        colorClasses = ' from-success-start to-success-end'; break;

                    case 1:
                        colorClasses = ' from-info-start to-info-end'; break;

                    case 2:
                        colorClasses = ' from-alert-start to-alert-end'; break;

                    default:
                        colorClasses = ' from-gray-semimedium to-gray-medium'; break;
                }

                temp_row.push('<td><div class="w-full h-full flex items-center justify-center"><a class="text-xs py-0.5 px-2 text-white-snow font-medium rounded-lg text-white-snow border-0 bg-gradient-to-r ' + colorClasses + '">' + this.$t('general.' + state[0]) + '</a></div></td>');

                if (temp_row.length > 1)
                    this.table.row.add(temp_row).draw(false);
            })

            if (Array.isArray(this.sensorDefault)) {
                setTimeout(() => {
                    this.sensorDefault.forEach( (sensor) => {
                        this.rechargeGraph(sensor, true)
                    })
                }, 250);
            } else {
                setTimeout(() => {
                    this.rechargeGraph(this.sensorDefault, true)
                }, 250);
            }
        },

        rechargeGraph(sensor, selector) {
            let chartData = new Array();
            let chartUnit = this.data.units[sensor];
            let chartName = this.data.names[sensor];

            this.data.epochMS.forEach( (date, index) => {
                if(typeof this.data.data[sensor][index] != 'undefined')
                    chartData.push([date, this.data.data[sensor][index]])
            })

            chartData.reverse();

            if(sensor == 4){

                let yAxisLabels = ['Water', 'Air']

                this.Charts.yAxis = (
                    {
                        id: 0,
                        opposite: true,
                        gridLineColor: '#647F8C',
                        title: {
                            text: ''
                        },
                        labels: {
                            formatter: function () {
                                // Utiliza las etiquetas personalizadas en lugar de la posición del tick
                                return yAxisLabels[this.pos];
                            },
                            style: {
                                color: 'rgba(200, 200, 200, 0.9)',
                                opacity: 0.7,
                            }
                        },
                        visible: true,
                    }
                );
            } else {
                this.Charts.yAxis = (
                    {
                        id: 0,
                        opposite: true,
                        gridLineColor: '#647F8C',
                        title: {
                            text: ''
                        },
                        labels: {
                            formatter: function () {
                                return this.pos + ' ' + chartUnit
                            },
                            style: {
                                color: 'rgba(200, 200, 200, 0.9)',
                                opacity: 0.7,
                            }
                        },
                        visible: true,
                    }
                );
            }

            this.Charts.tooltip.valueSuffix = chartUnit;

            this.Charts.series = [];

            if(sensor == 4){

                this.Charts.series.push({
                    name: chartName,
                    type: 'areaspline',
                    yAxis: 0,
                    data: chartData.filter(point => point !== undefined && point !== null),
                    color: "#05fff3",
                    lineWidth: 1.5,
                    zIndex: 1,
                    marker: {
                        enabled: false
                    },
                    tooltip: {
                        valueSuffix: chartUnit
                    }
                });

            } else {

                this.Charts.series.push({
                    name: chartName,
                    type: 'spline',
                    yAxis: 0,
                    data: chartData.filter(point => point !== undefined && point !== null),
                    color: "#05fff3",
                    lineWidth: 1.5,
                    zIndex: 1,
                    marker: {
                        enabled: false
                    },
                    tooltip: {
                        valueSuffix: chartUnit
                    }
                });
            }

            if(selector) {
                if(document.getElementById("id_chart__" + sensor)) {
                    Highcharts.chart("id_chart__" + sensor, this.Charts);
                }
            } else {
                if(document.getElementById("id_chart")) {
                    Highcharts.chart("id_chart", this.Charts);
                }
            }
        },

        validarRangoHorarioStart(){

            if(this.start_at.hour < 0 || this.start_at.hour === null || this.start_at.hour === undefined){
                this.start_at.hour = 0
            }

            if(this.start_at.hour > 23){
                this.start_at.hour = 23
            }

            if(document.getElementById('id_start_at')){
                document.getElementById('id_start_at').value = this.start_at.hour
            }

            this.validarHoraLocal()
        },

        validarRangoHorarioEnd(){

            if(this.end_at.hour < 0 || this.end_at.hour === null || this.end_at.hour === undefined){
                this.end_at.hour = 0
            }

            if(this.end_at.hour > 23){
                this.end_at.hour = 23
            }

            if(document.getElementById('id_end_at')){
                document.getElementById('id_end_at').value = this.end_at.hour
            }

            this.validarHoraLocal()
        },

        validarRangoHorario() {
            setTimeout(() => {
                this.validarRangoHorarioStart()
                this.validarRangoHorarioEnd()
            }, 0);
        },

        validarHoraLocal(){
            const timezoneOffset = (new Date()).getTimezoneOffset() / 60;
            let startHour, endHour

            if(
                this.start_at.hour !== null && this.start_at.hour !== undefined && this.start_at.hour !== '' &&
                this.end_at.hour !== null && this.end_at.hour !== undefined && this.end_at.hour !== ''
            ){

                if(this.start_at.hour + timezoneOffset < 0){
                    startHour = this.start_at.hour + timezoneOffset + 24
                } else if(this.start_at.hour + timezoneOffset > 23){
                    startHour = this.start_at.hour + timezoneOffset - 24
                } else {
                    startHour = this.start_at.hour + timezoneOffset
                }

                if(this.end_at.hour + timezoneOffset < 0){
                    endHour = this.end_at.hour + timezoneOffset + 24
                } else if(this.end_at.hour + timezoneOffset > 23){
                    endHour = this.end_at.hour + timezoneOffset - 24
                } else {
                    endHour = this.end_at.hour + timezoneOffset
                }

                this.disabledButtonEditFastMode = startHour >= endHour
            } else {
                this.disabledButtonEditFastMode = true
            }
        },

        methodUpdateDate(modelData) { //

            if( modelData[0].getFullYear() === modelData[1].getFullYear() &&
                modelData[0].getMonth() === modelData[1].getMonth() &&
                modelData[0].getDate() === modelData[1].getDate()){

                    modelData[0].setHours(0);
                    modelData[0].setMinutes(0);
                    modelData[0].setSeconds(0);

                    modelData[1].setHours(23);
                    modelData[1].setMinutes(59);
                    modelData[1].setSeconds(59);
            }

            let date_start = this.formatDate(modelData[0], 1);
            let date_end = this.formatDate(modelData[1], 1);

            axiosApiCalls.methodGet('/gateway/values/period/'+this.user.id+'/'+this.data.device.id+'/'+date_start+'/'+date_end, this.user.token)
            .then((response) => {
                    if(response.dates.length == 0){
                        window.emitter.emit(
                            'AppLayout_alert',
                            true,
                            this.$t('general.Gateway without records'),
                            this.$t('text.lacks records period'),
                            '18rem'
                        )
                    }

                    this.data.data = response.data;
                    this.data.dates = response.dates;
                    this.data.epochMS = response.epochMS;

                    this.rechargePage();
                return response;
            })
        },

        customFormat(date) {
            const year = date.getFullYear();
            const month = (date.getMonth() + 1).toString().padStart(2, '0');
            const day = date.getDate().toString().padStart(2, '0');
            const hour = date.getHours().toString().padStart(2, '0');

            return `${year}/${month}/${day}`;
            // return `${year}-${month}-${day} ${hour}:00`;
        },

        methodFormatDatetoDatepicker() {
            return generateDateDatepicker(this.date)
        },

        generateDateDatepicker(date) {

            var day1 = date[0].getDate();
            var month1 = date[0].getMonth() + 1;
            var year1 = date[0].getFullYear();
            var hour1 = date[0].getHours();
            var min1 = date[0].getMinutes();

            if(month1.toString().length == 1) month1 = '0' + month1;
            if(day1.toString().length == 1) day1 = '0' + day1;
            if(hour1.toString().length == 1) hour1 = '0' + hour1;
            if(min1.toString().length == 1) min1 = '0' + min1;

            var day2 = date[1].getDate();
            var month2 = date[1].getMonth() + 1;
            var year2 = date[1].getFullYear();
            var hour2 = date[1].getHours();
            var min2 = date[1].getMinutes();

            if(month2.toString().length == 1) month2 = '0' + month2;
            if(day2.toString().length == 1) day2 = '0' + day2;
            if(hour2.toString().length == 1) hour2 = '0' + hour2;
            if(min2.toString().length == 1) min2 = '0' + min2;

            return `${year1}/${month1}/${day1} ${hour1}:${min1} `+ this.$t("general.To") +` ${year2}/${month2}/${day2} ${hour2}:${min2}`;

        },

        mqtt_connection(iddevice) {

            var mqtt_topic = "/PUSHER/@P@" + this.user.idplatform + "/G@" + this.user.idgroup_selected + "/@D@" + iddevice;

            const mqtt = require('paho-mqtt');

            const mqtt_host = 'mqtt.datakorum.app';
            const mqtt_port = 8000;
            const mqtt_path = '';

            var userName = this.user.credentials.username;
            var password = this.user.credentials.password;

            var clientId = "APPclientid_" + parseInt(Math.random() * 100, 10);

            const client = new mqtt.Client(mqtt_host, mqtt_port, mqtt_path, clientId);

            client.connect({

            useSSL: true,
            mqttVersion: 4,
            userName: userName,
            password: password,

            onSuccess: () => {
                // función de éxito de conexión

                client.subscribe(mqtt_topic, {

                onSuccess: () => {
                    console.log('éxito de suscripción ' + iddevice)
                    // función de éxito de suscripción
                },

                onFailure: () => {
                    // función de fallo de suscripción
                    console.error('fallo de suscripción ' + iddevice)
                }

                });
            },

            onFailure: () => {
                // función de fallo de conexión
                console.error('fallo de conexión ' + iddevice)
            }

            });

            client.onConnectionLost = () => {
                // función que se ejecuta cuando perdemos la conexión
                console.error('perdida de conexión ' + iddevice)

                setTimeout(() => {
                    this.mqtt_connection(iddevice)
                }, 5000);

            };

            client.onMessageArrived = (message) => {
                // función que se ejecuta cuando llega un mensaje al topic suscrito

                var data = JSON.parse(JSON.parse(message.payloadString).frame_data);
                var when = this.formatDate(JSON.parse(message.payloadString).when * 1000);

                //this.rechargePage();

            };
        },

        getState(key) {
            if (key === undefined) {
                key = 0
            }

            let dmax = (typeof this.dmax != 'undefined') ? parseFloat(this.dmax) : 109;
            let dmin = (typeof this.dmin != 'undefined') ? parseFloat(this.dmin) : 71;
            let margin = (typeof this.margin != 'undefined') ? parseFloat(this.margin) : 5;

            if (
                typeof this.data.data[4][key] != "undefined" &&
                typeof this.data.data[2][key] != "undefined" &&
                typeof this.data.data[3][key] != "undefined"
            ) {
                // Tenemos medidas para esa key

                if (this.data.data[4][key] == 1) {
                    // AIR

                    if (this.data.data[3][key] < dmin - margin / 2) {
                        // Dist < dmin - m/2

                        // Err_S
                        return 7;
                    } else if (this.data.data[3][key] <= dmin + margin / 2) {
                        // dmin - m/2 <= Dist <= dmin + m/2

                        // Caso indeterminado 5 -> Err_S
                        return 7;
                    } else if (this.data.data[3][key] < dmax - margin / 2) {
                        // dmin + m/2 < Dist < dmax - m/2

                        // Caso indeterminado 2
                        if (typeof this.data.data[4][key + 1] != "undefined") {
                            // Tenemos medida anterior

                            if (this.data.data[3][key] < this.data.data[4][key + 1]) {
                                // Dist con tendencia descendiente

                                // VP2
                                return 1;
                            } else {
                                // Dist con tendencia ascendiente

                                // Err_F
                                return 5;
                            }
                        } else if (typeof this.data.data[4][key - 1] != "undefined") {
                            // Tenemos medida posterior

                            if (this.data.data[4][key - 1] < this.data.data[3][key]) {
                                // Dist con tendencia descendiente

                                // VP2
                                return 1;
                            } else {
                                // Dist con tendencia ascendiente

                                // Err_F
                                return 5;
                            }
                        } else {
                            // Sin medida anterior o posterior -> Error

                            // Err_S
                            return 7;
                        }
                    } else if (this.data.data[3][key] <= dmax + margin / 2) {
                        // dmax - m/2 <= Dist <= dmax + m/2

                        // Caso indeterminado 3 -> VP1 con prioridad frente a A
                        return 0;
                    } else {
                        // dmax + m/2 < Dist

                        // Err_S
                        return 7;
                    }
                } else if (this.data.data[4][key] == 0) {
                    // WATER

                    if (this.data.data[3][key] < dmin - margin / 2) {
                        // Dist < dmin - m/2

                        if (this.data.data[2][key] < 2) {
                            // Err_S
                            return 7;
                        } else {
                            // Caso indeterminado 1 -> Err_L con prioridad frente a Err_S
                            return 6;
                        }
                    } else if (this.data.data[3][key] <= dmin + margin / 2) {
                        // dmin - m/2 <= Dist <= dmin + m/2

                        // Caso indeterminado 4 -> VC y VP3 indistinguibles, VC con prioridad
                        return 4;
                    } else if (this.data.data[3][key] < dmax - margin / 2) {
                        // dmin + m/2 < Dist < dmax - m/2

                        // VP2
                        return 1;
                    } else if (this.data.data[3][key] <= dmax + margin / 2) {
                        // dmax - m/2 <= Dist <= dmax + m/2

                        // A
                        return 3;
                    } else {
                        // dmax + m/2 < Dist

                        // Err_S
                        return 7;
                    }
                } else {
                    // !(AIR/WATER)

                    // Err_S
                    return 7;
                }
            } else {
                // Err_S
                return 7;
            }
        },

        getStateName(key, getColor) {

            let idState

            if (key === undefined) {
                idState = this.idState
            } else {
                idState = this.getState(key)
            }

            if (getColor) {
                switch (idState) {
                    case 0:
                        return ['VENTING PHASE 1', 0]

                    case 1:
                        return ['VENTING PHASE 2', 0]

                    case 2:
                        return ['VENTING PHASE 3', 0]

                    case 3:
                        return ['AERATION', 0]

                    case 4:
                        return ['VALVE CLOSED', 1]

                    case 5:
                        return ['FLOAT HANG UP', 2]

                    case 6:
                        return ['VALVE LEAKING', 2]

                    case 7:
                        return ['SENSOR DEFECT', 2]

                    default:
                        return ['SENSOR DEFECT', 2]
                }
            } else {
                switch (idState) {
                    case 0:
                        this.idColor = 0
                        return 'VENTING PHASE 1'

                    case 1:
                        this.idColor = 0
                        return 'VENTING PHASE 2'

                    case 2:
                        this.idColor = 0
                        return 'VENTING PHASE 3'

                    case 3:
                        this.idColor = 0
                        return 'AERATION'

                    case 4:
                        this.idColor = 1
                        return 'VALVE CLOSED'

                    case 5:
                        this.idColor = 2
                        return 'FLOAT HANG UP'

                    case 6:
                        this.idColor = 2
                        return 'VALVE LEAKING'

                    case 7:
                        this.idColor = 2
                        return 'SENSOR DEFECT'

                    default:
                        this.idColor = 2
                        return 'SENSOR DEFECT'
                }
            }
        },

        getStateDescription() {

            switch (this.idState) {
                case 0:
                    return 'Releases air 1'

                case 1:
                    return 'Releases air 2'

                case 2:
                    return 'Releases air 3'

                case 3:
                    return 'Air to enter the pipeline'

                case 4:
                    return 'Seals the pipeline'

                case 5:
                    return 'Error: Float'

                case 6:
                    return 'Error: Leaking'

                case 7:
                    return 'Error: Sensors'

                default:
                    return 'Error: Sensors'
            }
        },

        setElapsedTime() {
            this.elapsedTime = (this.data.dates != undefined && this.data.dates[0]) ? this.getElapsedTime(this.data.dates[0], new Date()) : '0 s';
        },

        getDistanceColor(key) {

            let dmax = (typeof this.dmax != 'undefined') ? parseFloat(this.dmax) : 109;
            let dmin = (typeof this.dmin != 'undefined') ? parseFloat(this.dmin) : 71;
            let margin = (typeof this.margin != 'undefined') ? parseFloat(this.margin) : 5;

            if (this.data.data[3][0] >= key) {
                if (key < dmin - (margin/2)) {
                    return '#05fff350'
                } else if (key <= dmin + (margin/2)) {
                    return '#05fff3CC'
                } else if (key < dmax - (margin/2)) {
                    return '#05fff3A0'
                } else if (key <= dmax + (margin/2)) {
                    return '#05fff3CC'
                } else {
                    return '#05fff350'
                }
            } else /*if (Math.round(this.data.data[4][0]) == key) {
                return '#05fff3'
            } else*/ {
                return 'rgba(100, 100, 100, 0.4)'
            }
        },

        getDistanceRange(distance) {

            let dmax = (typeof this.dmax != 'undefined') ? parseFloat(this.dmax) : 109;
            let dmin = (typeof this.dmin != 'undefined') ? parseFloat(this.dmin) : 71;
            let margin = (typeof this.margin != 'undefined') ? parseFloat(this.margin) : 5;

            if (distance < dmin - (margin/2)) {
                return 'D<dmin '
            } else if (distance <= dmin + (margin/2)) {
                return 'D=dmin '
            } else if (distance < dmax - (margin/2)) {
                return 'dmin<D<dmax'
            } else if (distance <= dmax + (margin/2)) {
                return 'D=dmax '
            } else {
                return 'D>dmax '
            }
        },

        enableDistanceButton(){

            if (
                (document.getElementById('idDmin') && typeof document.getElementById('idDmin').value != 'undefined' && parseInt(this.dmin) != parseInt(document.getElementById('idDmin').value)) ||
                (document.getElementById('idDmax') && typeof document.getElementById('idDmax').value != 'undefined' && parseInt(this.dmax) != parseInt(document.getElementById('idDmax').value)) ||
                (document.getElementById('idmargin') && typeof document.getElementById('idmargin').value != 'undefined' && parseInt(this.margin) != parseInt(document.getElementById('idmargin').value))
            ) {
                this.enableDistButton = true;
                return true;
            }

            this.enableDistButton = false;
            return false;
        },

        putDistances(){
            let data = {
                dmin: document.getElementById('idDmin').value,
                dmax: document.getElementById('idDmax').value,
                margin: document.getElementById('idmargin').value,
            };

            axiosApiCalls.methodPut('/gateway/edit_descriptors/'+this.user.id+'/'+this.data.device.id, this.user.token, data)
            .then((response) => {
                this.dmin = data.dmin
                this.dmax = data.dmax
                this.margin = data.margin

                this.distancesModal = false

                return response;
            })

            this.rechargePage()
        },

        putFastMode() {
            const timezoneOffset = (new Date()).getTimezoneOffset() / 60;
            let startHour, endHour

            if(
                this.start_at.hour !== null && this.start_at.hour !== undefined && this.start_at.hour !== '' &&
                this.end_at.hour !== null && this.end_at.hour !== undefined && this.end_at.hour !== ''
            ){

                if(this.start_at.hour + timezoneOffset < 0){
                    startHour = this.start_at.hour + timezoneOffset + 24
                } else if(this.start_at.hour + timezoneOffset > 23){
                    startHour = this.start_at.hour + timezoneOffset - 24
                } else {
                    startHour = this.start_at.hour + timezoneOffset
                }

                if(this.end_at.hour + timezoneOffset < 0){
                    endHour = this.end_at.hour + timezoneOffset + 24
                } else if(this.end_at.hour + timezoneOffset > 23){
                    endHour = this.end_at.hour + timezoneOffset - 24
                } else {
                    endHour = this.end_at.hour + timezoneOffset
                }

                if(startHour < endHour) {
                    const selectedDate = new Date(this.selectedDate)

                    if((this.start_at.hour + timezoneOffset) < 0){
                        selectedDate.setDate(selectedDate.getDate() - 1);
                    }

                    const day = selectedDate.getDate().toString().padStart(2, '0');
                    const month = (selectedDate.getMonth() + 1).toString().padStart(2, '0');

                    const selectedDateFormatted = `${day}-${month}`;

                    var paramFastMode =  startHour + ',' +  endHour + ',' + '30' + ';' + selectedDateFormatted + ';' + '15';

                    let data = {
                        srt2:  paramFastMode
                    };

                    axiosApiCalls.methodPut('/gateway/edit_parameters/'+this.user.id+'/'+this.data.device.id, this.user.token, data)
                    .then((response) => {
                        this.closeFastMode();
                        setTimeout(() => {
                            this.rechargePage();
                        }, 0);
                    })
                }
            }
        },

        enableMaintenanceButton() {

            if (
                (document.getElementById('idYear') && typeof document.getElementById('idYear').value != 'undefined' && parseInt(this.maintenanceYear) != parseInt(document.getElementById('idYear').value)) ||
                (document.getElementById('idMonth') && typeof document.getElementById('idMonth').value != 'undefined' && parseInt(this.maintenanceMonth) != parseInt(document.getElementById('idMonth').value)) ||
                (document.getElementById('idDay') && typeof document.getElementById('idDay').value != 'undefined' && parseInt(this.maintenanceDay) != parseInt(document.getElementById('idDay').value))
            ) {
                this.enableMaintButton = true;
                return true;
            }

            this.enableMaintButton = false;
            return false;
        },

        cancelMaintenanceDate(){
            this.maintenanceDay = String(this.data.device.descriptors.maintenanceDay ? this.data.device.descriptors.maintenanceDay : 1).padStart(2, '0');
            this.maintenanceMonth = String(this.data.device.descriptors.maintenanceMonth ? this.data.device.descriptors.maintenanceMonth : 1).padStart(2, '0');
            this.maintenanceYear = String(this.data.device.descriptors.maintenanceYear ? this.data.device.descriptors.maintenanceYear : 2000).padStart(4, '0');

            document.getElementById('idYear').value = this.maintenanceYear;
            document.getElementById('idMonth').value = this.maintenanceMonth;
            document.getElementById('idDay').value = this.maintenanceDay;

            this.enableMaintenanceButton()
        },

        putMaintenanceDate() {

            let data = {
                maintenanceYear: document.getElementById('idYear').value,
                maintenanceMonth: document.getElementById('idMonth').value,
                maintenanceDay: document.getElementById('idDay').value,
            };

            axiosApiCalls.methodPut('/gateway/edit_descriptors/'+this.user.id+'/'+this.data.device.id, this.user.token, data)
            .then((response) => {
                this.maintenanceYear = data.maintenanceYear
                this.maintenanceMonth = data.maintenanceMonth
                this.maintenanceDay = data.maintenanceDay

                return response;
            })
        },

        getContainerSize() {
            var myContainer = document.getElementById('myContainer');

            if(myContainer){
                var width = myContainer.clientWidth;
                var height = myContainer.clientHeight;
            }
            this.containerSize = String(Math.min(width, height));
        },

        openDistances(){
            this.distancesModal = true
        },

        closeDistances(){
            this.distancesModal = false
        },

        openFastMode(){
            this.selectedDate = new Date ();
            this.validarRangoHorario();
            this.fastMode = true
        },

        closeFastMode(){
            this.fastMode = false
        },

        adminLvl(rol){

            // if(rol == 'admin') {
            //     return false;
            // } else {
            //     return true;
            // }

            switch(rol){
                case 'admin': return this.user.admin == 0 || this.user.admin == 1 || this.user.admin == 2;

                case 'user': return this.user.admin == 3;

                default: return false;
            }

        }
    },
}
</script>
